@@ -98,6 +98,8 @@ static GtkAction *pause_action;
 static GtkAction *end_game_action;
 static GtkAction *preferences_action;
 static GtkAction *scores_action;
+static GtkAction *fullscreen_action;
+static GtkAction *leave_fullscreen_action;
 
 static void
 hide_cursor (void)
@@ -108,6 +110,36 @@ hide_cursor (void)
 	}
 }
 
+static void 
+set_fullscreen_actions (gboolean is_fullscreen)
+{
+        gtk_action_set_sensitive (leave_fullscreen_action, is_fullscreen);
+        gtk_action_set_visible (leave_fullscreen_action, is_fullscreen);
+
+        gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
+        gtk_action_set_visible (fullscreen_action, !is_fullscreen);
+}
+
+static void 
+fullscreen_cb (GtkAction *action)
+{
+	if (action == fullscreen_action) {
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	} else {
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+	}
+}
+
+static void 
+window_state_cb (GtkWidget *widget, GdkEventWindowState *event)
+{
+	/* Handle fullscreen, in case something else takes us to/from fullscreen.*/
+	if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
+        	set_fullscreen_actions (event->new_window_state 
+					& GDK_WINDOW_STATE_FULLSCREEN);
+}
+
+
 static void
 show_cursor (void)
 {
@@ -601,6 +633,7 @@ help_cb (GtkAction *action, gpointer data)
  
 static const GtkActionEntry action_entry[] = {
 	{ "GameMenu", NULL, N_("_Game") },
+        { "ViewMenu", NULL, N_("_View") },
 	{ "SettingsMenu", NULL, N_("_Settings") },
 	{ "HelpMenu", NULL, N_("_Help") },
 	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game_cb) },
@@ -609,6 +642,8 @@ static const GtkActionEntry action_entry[] = {
 	{ "EndGame", GAMES_STOCK_END_GAME, NULL, NULL, NULL, G_CALLBACK (end_game_cb) },
 	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (show_scores_cb) },
 	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_cb) },
+        { "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_cb) },
+        { "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_cb) },
 	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (gnibbles_preferences_cb) },
 	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_cb) },
 	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_cb) }
@@ -628,6 +663,10 @@ static const char ui_description[] =
 "      <separator/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menuitem action='Fullscreen'/>"
+"      <menuitem action='LeaveFullscreen'/>"
+"    </menu>"
 "    <menu action='SettingsMenu'>"
 "      <menuitem action='Preferences'/>"
 "    </menu>"
@@ -656,6 +695,10 @@ create_menus (GtkUIManager *ui_manager)
         end_game_action = gtk_action_group_get_action (action_group, "EndGame");
         pause_action = gtk_action_group_get_action (action_group, "Pause");
 	preferences_action = gtk_action_group_get_action (action_group, "Preferences");
+	fullscreen_action = gtk_action_group_get_action (action_group, "Fullscreen");
+	leave_fullscreen_action = gtk_action_group_get_action (action_group,
+							       "LeaveFullscreen");
+
 }
 
 static void
@@ -675,6 +718,9 @@ setup_window (void)
 			G_CALLBACK (quit_cb), NULL);
 	g_signal_connect (G_OBJECT (window), "delete_event",
 			G_CALLBACK (delete_cb), NULL);
+	g_signal_connect (G_OBJECT (window), "window_state_event",
+                    	G_CALLBACK (window_state_cb), NULL);
+
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gnome_app_set_contents (GNOME_APP (window), vbox);
@@ -682,6 +728,7 @@ setup_window (void)
 	games_stock_init ();
 	ui_manager = gtk_ui_manager_new ();
 	create_menus (ui_manager);
+	set_fullscreen_actions (FALSE);
 
 	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
@@ -724,6 +771,8 @@ setup_window (void)
 				     BOARDHEIGHT*5);
 	g_signal_connect (G_OBJECT (drawing_area), "expose_event",
 			G_CALLBACK (expose_event_cb), NULL);
+	/* We do our own double-buffering. */
+	gtk_widget_set_double_buffered(GTK_WIDGET (drawing_area), FALSE);
 	gtk_widget_set_events (drawing_area, GDK_BUTTON_PRESS_MASK |
 			GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK);
 	gtk_widget_show (drawing_area);
